package Applet;


import DiffieHellman.*;
import javacard.framework.*;
import javacard.security.*;
import javacardx.crypto.*;
import static jpass.smartcard.TerminalManager.byteToHex;

public class JPassApplet extends javacard.framework.Applet
{
    private static final byte DEFAULT_USER_PIN[] = {(byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04};
       
    final static short ARRAY_LENGTH                  = (short) 0xff;
    
    // MAIN INSTRUCTION CLASS
    final static byte CLA_SIMPLEAPPLET                = (byte) 0xB0;
    
    final static byte INS_VERIFYPIN                  = (byte) 0x11;
    final static byte INS_SETPIN                     = (byte) 0x12;
    final static byte INS_ISPINUNLOCKED              = (byte) 0x13;
    
    final static byte INS_CBC_BULK_INIT              = (byte) 0x50;
    final static byte INS_CBC_BULK_PROCESS           = (byte) 0x51;
    final static byte INS_CBC_BULK_FINISH            = (byte) 0x52;
    
    final static short SW_OBJECT_NOT_AVAILABLE       = (short) 0x6711;
    final static short SW_BAD_PIN                    = (short) 0x6900;
    final static short SW_PIN_LOCKED                 = (short) 0x6901;
    final static short SW_INIT_ERROR                 = (short) 0x6911;
    
    final static short SW_BULK_CBC_IV_LENGTH_BAD     = (short) 0x5111;
    final static short SW_BULK_CBC_BLOCK_LENGTH_BAD  = (short) 0x5112;
    
    final static short SW_OTP_KEY_LENGTH_BAD         = (short) 0x2111;      
    
    
    final static short CBC_MODE_ENCRYPT              = (short) 0x01;
    final static short CBC_MODE_DECRYPT              = (short) 0x02;
    
    final static short AES_BLOCK_LENGTH              = (short)16;
    
    final static short SW_Exception                     = (short) 0xff01;
    final static short SW_ArrayIndexOutOfBoundsException = (short) 0xff02;
    final static short SW_ArithmeticException           = (short) 0xff03;
    final static short SW_ArrayStoreException           = (short) 0xff04;
    final static short SW_NullPointerException          = (short) 0xff05;
    final static short SW_NegativeArraySizeException    = (short) 0xff06;
    final static short SW_CryptoException_prefix        = (short) 0xf100;
    final static short SW_SystemException_prefix        = (short) 0xf200;
    final static short SW_PINException_prefix           = (short) 0xf300;
    final static short SW_TransactionException_prefix   = (short) 0xf400;
    final static short SW_CardRuntimeException_prefix   = (short) 0xf500;   
    
    // PIN support
    private   RandomData     m_secureRandom = null;
    private   OwnerPIN       m_pin = null;
    
    // Encryption support
    private   AESKey         m_aesKey = null;
    private   Cipher         m_bulk_cbc_cipher = null;
    private   short          m_aes_mode = 0;   
    private   byte           m_last_block[] = null;
    private   short          m_last_block_length = 0;
    PKCS5Padding             padding = null;
    
    // TEMPORARY ARRAY IN RAM
    private   byte        m_ramArray[] = null;
    // PERSISTENT ARRAY IN EEPROM
    private   byte        m_dataArray[] = null;
    
    // SecureChannel
    private SecureChannel secureChannel;
    
    
    // dh
    
   final static byte INS_DH_INIT                  = (byte) 0x60;
   final static byte INS_DH_GET_Y_PART            = (byte) 0x61;
   final static byte INS_DH_GET_Y_PART2           = (byte) 0x65;
   final static byte INS_DH_SET                   = (byte) 0x62;
   final static byte INS_DH_FINAL                 = (byte) 0x63;
   
   private final AESKey encKey = (AESKey) KeyBuilder.buildKey(KeyBuilder.TYPE_AES_TRANSIENT_RESET, KeyBuilder.LENGTH_AES_128, false);
   private final Cipher aesCipher = Cipher.getInstance(Cipher.ALG_AES_BLOCK_128_ECB_NOPAD, false);
   
   public DiffieHellman dh;
   
   public final byte[] reply = {(byte) 0x48, (byte) 0x65, (byte) 0x6c, (byte) 0x6c,
        (byte) 0x6f, (byte) 0x20, (byte) 0x4a, (byte) 0x61, (byte) 0x76, (byte) 0x61,
(byte) 0x20, (byte) 0x43, (byte) 0x61, (byte) 0x72, (byte) 0x64, (byte) 0x2e};

    protected JPassApplet(byte[] buffer, short offset, byte length)
    {
        // data offset is used for application specific parameter.
        // initialization with default offset (AID offset).
        short dataOffset = offset;

        if(length > 9) {
            dataOffset += (short)( 1 + buffer[offset]);
            dataOffset += (short)( 1 + buffer[dataOffset]);
            dataOffset++;

            // Allocate arrays
            m_dataArray = new byte[ARRAY_LENGTH];
            Util.arrayFillNonAtomic(m_dataArray, (short) 0, ARRAY_LENGTH, (byte) 0);
            m_ramArray = JCSystem.makeTransientByteArray((short) 260, JCSystem.CLEAR_ON_DESELECT);
            m_last_block = JCSystem.makeTransientByteArray((short) 260, JCSystem.CLEAR_ON_DESELECT);
            
            // Setup crypto            
            m_bulk_cbc_cipher = Cipher.getInstance(Cipher.ALG_AES_BLOCK_128_CBC_NOPAD, false);
            m_aesKey = (AESKey) KeyBuilder.buildKey(KeyBuilder.TYPE_AES, KeyBuilder.LENGTH_AES_256, false);
            m_aesKey.setKey(m_ramArray, (short) AES_BLOCK_LENGTH);
            padding = new PKCS5Padding(AES_BLOCK_LENGTH);
            
            // Setup PIN
            m_pin = new OwnerPIN((byte) 5, (byte) 4);
            m_pin.update(DEFAULT_USER_PIN, (byte) 0, (byte) 4);
            
            // Setup random generation
            m_secureRandom = RandomData.getInstance(RandomData.ALG_SECURE_RANDOM);
            
            secureChannel = new SecureChannel();
            byte[] enc = {(byte) 0x10, (byte) 0x02, (byte) 0x03, (byte) 0x04, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04};
            secureChannel.setSessionKey(enc, (short)0);
            
        }
        dh = new DiffieHellman();
        register();
        
    }

    /**
     * Method installing the applet.
     * @param bArray the array constaining installation parameters
     * @param bOffset the starting offset in bArray
     * @param bLength the length in bytes of the data parameter in bArray
     */
    public static void install(byte[] bArray, short bOffset, byte bLength) throws ISOException
    {
        // applet  instance creation 
        new JPassApplet (bArray, bOffset, bLength);
    }

    /**
     * Select method returns true if applet selection is supported.
     * @return boolean status of selection.
     */
    public boolean select()
    {
        return true;
    }

    /**
     * Deselect method called by the system in the deselection process.
     */
    public void deselect()
    {
        return;
    }

    /**
     * Method processing an incoming APDU.
     * @see APDU
     * @param apdu the incoming APDU
     * @exception ISOException with the response bytes defined by ISO 7816-4
     */
    
    public void process(APDU apdu) throws ISOException{
        try {
            byte[] apduBuffer = apdu.getBuffer();
            byte ins = apduBuffer[ISO7816.OFFSET_INS];
            
            if(ins == INS_DH_INIT || ins == INS_DH_GET_Y_PART || ins == INS_DH_SET || ins == INS_DH_FINAL || ins == INS_DH_GET_Y_PART2) {
                switch(ins) {
                    case INS_DH_INIT: dh.init(); break;
                    case INS_DH_GET_Y_PART: getY(apdu); break;
                    case INS_DH_GET_Y_PART2: getY2(apdu); break;
                    case INS_DH_SET: setY(apdu); break;
                    case INS_DH_FINAL: dh.doFinal(encKey); break;
                    default :  ISOException.throwIt( ISO7816.SW_INS_NOT_SUPPORTED ) ; break;
                }
            }else{
                secureChannel.unwrapCommandAPDU();
                this.processDecrypted(apdu);
            }
            
            
            
        } catch (ISOException e) {
            throw e;
        }
    }
    
    public static String bytesToHex(byte[] data) {
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < data.length; i++) {
            buf.append(byteToHex(data[i]));
            buf.append(" ");
        }
        return (buf.toString());
    }
    
    private void wrapAndSend(APDU apdu, short length ) {
        short wrapResponseAPDU = secureChannel.wrapResponseAPDU(ISO7816.OFFSET_CDATA, length);
        apdu.setOutgoingAndSend(ISO7816.OFFSET_CDATA, wrapResponseAPDU);
    }
    
    private void testwrapAndSend(APDU apdu, short length ) {
        System.out.println("wut meit");
        short wrapResponseAPDU = secureChannel.wrapResponseAPDU((short) 0, length);
        System.out.println("wut meit");
        apdu.setOutgoingAndSend(ISO7816.OFFSET_CDATA, wrapResponseAPDU);
    }
    
    public void processDecrypted(APDU apdu) throws ISOException
    {
        byte[] apduBuffer = apdu.getBuffer();
        
        // ignore the applet select command dispached to the process
        if (selectingApplet())
            return;

        try {
            // APDU instruction parser
            if (apduBuffer[ISO7816.OFFSET_CLA] == CLA_SIMPLEAPPLET) {
                byte ins = apduBuffer[ISO7816.OFFSET_INS];
                if(ins == INS_VERIFYPIN) {
                    if((short)m_pin.getTriesRemaining() > (short)0) {
                       VerifyPIN(apdu); 
                    } else {
                        ISOException.throwIt(SW_BAD_PIN);
                    }
                } else if (ins == INS_ISPINUNLOCKED) {
                    isPinUnlocked(apdu);
                } else {
                    if(m_pin.isValidated()) {
                        switch(ins) {
                            case INS_SETPIN: SetPIN(apdu); break;
                            case INS_CBC_BULK_INIT: CBC_BULK_Init(apdu); break;
                            case INS_CBC_BULK_PROCESS: CBC_BULK_Process(apdu); break;
                            case INS_CBC_BULK_FINISH: CBC_BULK_Finish(apdu); break;
                            case INS_DH_INIT: dh.init(); break;
                            case INS_DH_GET_Y_PART: getY(apdu); break;
                            default :  ISOException.throwIt( ISO7816.SW_INS_NOT_SUPPORTED ) ; break;
                        }      
                    } else {
                        ISOException.throwIt(SW_BAD_PIN);
                    }
                }
            }
        else ISOException.throwIt(ISO7816.SW_CLA_NOT_SUPPORTED);
        } catch (ISOException e) {
            throw e; // Our exception from code, just re-emit
        } catch (ArrayIndexOutOfBoundsException e) {
            ISOException.throwIt(SW_ArrayIndexOutOfBoundsException);
        } catch (ArithmeticException e) {
            ISOException.throwIt(SW_ArithmeticException);
        } catch (ArrayStoreException e) {
            ISOException.throwIt(SW_ArrayStoreException);
        } catch (NullPointerException e) {
            ISOException.throwIt(SW_NullPointerException);
        } catch (NegativeArraySizeException e) {
            ISOException.throwIt(SW_NegativeArraySizeException);
        } catch (CryptoException e) {
            ISOException.throwIt((short) (SW_CryptoException_prefix | e.getReason()));
        } catch (SystemException e) {
            ISOException.throwIt((short) (SW_SystemException_prefix | e.getReason()));
        } catch (PINException e) {
            ISOException.throwIt((short) (SW_PINException_prefix | e.getReason()));
        } catch (TransactionException e) {
            ISOException.throwIt((short) (SW_TransactionException_prefix | e.getReason()));
        } catch (CardRuntimeException e) {
            ISOException.throwIt((short) (SW_CardRuntimeException_prefix | e.getReason()));
        } catch (Exception e) {
            ISOException.throwIt(SW_Exception);
        }
    }
    
    void getY(APDU apdu){
        byte[]    apdubuf = apdu.getBuffer();
        short     dataLen = apdu.setIncomingAndReceive();

        dh.getY(apdubuf, (short) 0);

        apdu.setOutgoingAndSend((short) 0, (short) 255);
//        apdu.sendBytesLong(apdubuf, (short) 0, DiffieHellman.maxLength);
    }
    
    void getY2(APDU apdu){
        byte[]    apdubuf = apdu.getBuffer();
        short     dataLen = apdu.setIncomingAndReceive();

        dh.getY(apdubuf, (short) 0);

        Util.arrayCopyNonAtomic(apdubuf, (short)255, apdubuf, (short) 0, (short) 1);   
        apdu.setOutgoingAndSend((short) 0, (short) 1);
//        apdu.sendBytesLong(apdubuf, (short) 0, DiffieHellman.maxLength);
    }
    
    void setY(APDU apdu){
        byte[]    apdubuf = apdu.getBuffer();
        short     dataLen = apdu.setIncomingAndReceive();
        
        dh.setY(apdubuf, (short) 3, DiffieHellman.maxLength, (short) 0);
    }

            
    // VERIFY PIN
     void VerifyPIN(APDU apdu) {
      byte[]    apdubuf = apdu.getBuffer();
      short     dataLen = apdu.setIncomingAndReceive();

      if (m_pin.check(apdubuf, ISO7816.OFFSET_CDATA, (byte) dataLen) == false) { 
          ISOException.throwIt(SW_BAD_PIN);
      } else {
          m_pin.reset();
          m_pin.check(apdubuf, ISO7816.OFFSET_CDATA, (byte) dataLen);
      }
    }
     
     void isPinUnlocked(APDU apdu) {
         byte[] apdubuf = apdu.getBuffer();
         if (!m_pin.isValidated())  {
             ISOException.throwIt(SW_PIN_LOCKED);
         }
     }
     

     // SET PIN
     void SetPIN(APDU apdu) {
      byte[]    apdubuf = apdu.getBuffer();
      short     dataLen = apdu.setIncomingAndReceive();

      /*
       * Even though that this command would not be called if card is locked, we still add one more check if it was unlocked.
      */
      if (m_pin.isValidated())
           m_pin.update(apdubuf, ISO7816.OFFSET_CDATA, (byte) dataLen);
      else 
          ISOException.throwIt(SW_BAD_PIN);
    }
     
    void CBC_BULK_Init(APDU apdu) {
      byte[]        apdubuf = apdu.getBuffer();
      short         dataLen = apdu.setIncomingAndReceive();
      
      if ((short) dataLen != AES_BLOCK_LENGTH) ISOException.throwIt(SW_BULK_CBC_IV_LENGTH_BAD);
      short mode = apdubuf[ISO7816.OFFSET_P1];
      m_last_block_length = (short)0;
      
      switch (mode) {
            case CBC_MODE_ENCRYPT:
                m_aes_mode = Cipher.MODE_ENCRYPT;
                m_bulk_cbc_cipher.init(m_aesKey, Cipher.MODE_ENCRYPT, apdubuf, (short)ISO7816.OFFSET_CDATA, dataLen);                
                break;
            case CBC_MODE_DECRYPT:
                m_aes_mode = Cipher.MODE_DECRYPT;
                m_bulk_cbc_cipher.init(m_aesKey, Cipher.MODE_DECRYPT, apdubuf, (short)ISO7816.OFFSET_CDATA, dataLen);                
                break;
            default:
                ISOException.throwIt(SW_INIT_ERROR);
                break;
        }
    }
    
    /*
     *  For decryption, we run one block forward - when block n  comes, we decrypt stored (n-1) block and return it, and store block number n.
     *  In Finish method we unpad this block n and return it. Possibilty it will be even empty - if it was 0x16 16 times.
     * 
     *  For encryption, we process blocks as they come, and Finish method can return 0x16 16 times or nothing, if the last block was already padded. 
     */
    void CBC_BULK_Process(APDU apdu) {
        byte[]    apdubuf = apdu.getBuffer();
        short     dataLen = apdu.setIncomingAndReceive();
        
        
        if (m_aes_mode == Cipher.MODE_DECRYPT) {
            // Discard not padded blocks
            if ((short) (dataLen) != AES_BLOCK_LENGTH) ISOException.throwIt(SW_BULK_CBC_BLOCK_LENGTH_BAD);
            
            // Decrypt previously stored block and move it to ramArray
            short outLen = m_bulk_cbc_cipher.update(m_last_block, (short)0, m_last_block_length, m_ramArray, (short) 0); 
            
            // Store just arrived block (it will be processed later)
            Util.arrayCopyNonAtomic(apdubuf, ISO7816.OFFSET_CDATA, m_last_block, (short) 0, dataLen);
            m_last_block_length = dataLen;                    
                    
            // Send just decrypted block from ramArray
            Util.arrayCopyNonAtomic(m_ramArray, (short) 0, apdubuf, ISO7816.OFFSET_CDATA, outLen);
            this.wrapAndSend(apdu, outLen);
        }
        
        
        if (m_aes_mode == Cipher.MODE_ENCRYPT) {
            Util.arrayCopyNonAtomic(apdubuf, ISO7816.OFFSET_CDATA, m_last_block, (short) 0, dataLen);
            m_last_block_length = dataLen;
            
            if ((short) (dataLen) != AES_BLOCK_LENGTH) { // Pad the block if necessary
                short pad = padding.padLength(dataLen);
                padding.padWithLen(m_last_block, (short) dataLen, pad);
                dataLen = AES_BLOCK_LENGTH;
            }
            
            short outLen = m_bulk_cbc_cipher.update(m_last_block, (short)0, dataLen, m_ramArray, (short) 0); 
            Util.arrayCopyNonAtomic(m_ramArray, (short) 0, apdubuf, ISO7816.OFFSET_CDATA, outLen);
            this.wrapAndSend(apdu, outLen);
        }
    }     
    
    void CBC_BULK_Finish(APDU apdu) {
        byte[]    apdubuf = apdu.getBuffer();
        
        if (m_aes_mode == Cipher.MODE_DECRYPT) {
            // Decrypt last received block
            short outLen = m_bulk_cbc_cipher.doFinal(m_last_block, (short)0, m_last_block_length, m_ramArray, (short) 0);
        
            short unpad = padding.unpad(m_ramArray, (short)0, AES_BLOCK_LENGTH);
                if(unpad >= 0) {
                    outLen = unpad;
                }
            Util.arrayCopyNonAtomic(m_ramArray, (short) 0, apdubuf, ISO7816.OFFSET_CDATA, outLen);
            this.wrapAndSend(apdu, outLen);
        }
        
        if (m_aes_mode == Cipher.MODE_ENCRYPT) {
            // Add padded block 0x016 16 if necessary, otherwise send back empty block.            
            short outLen = 0;            
            if(m_last_block_length == AES_BLOCK_LENGTH ) {
                short pad = padding.padLength(AES_BLOCK_LENGTH);
                padding.padWithLen(m_last_block, (short) 0, pad);
                outLen = AES_BLOCK_LENGTH;
            }
            m_bulk_cbc_cipher.doFinal(m_last_block, (short)0, outLen, m_ramArray, (short) 0);
            Util.arrayCopyNonAtomic(m_ramArray, (short) 0, apdubuf, ISO7816.OFFSET_CDATA, outLen);
            
            this.wrapAndSend(apdu, outLen);
        }
    }
}




