/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package jpass.smartcard;

import javacard.framework.ISO7816;
import javacard.framework.Util;
import static jpass.smartcard.TerminalManager.byteToHex;

/**
 *
 * @author macbook
 */
public class APDUFactory {
    public static byte[] verifyPin(byte[] pinBytes) {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH + pinBytes.length];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x11;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x01;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte)pinBytes.length;

        Util.arrayCopyNonAtomic(pinBytes, (short) 0, apdu, ISO7816.OFFSET_CDATA, (byte)pinBytes.length);
        return apdu;
    }
    
    public static byte[] setPin(byte[] pinBytes) {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH + pinBytes.length];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x12;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x01;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte)pinBytes.length;

        Util.arrayCopyNonAtomic(pinBytes, (short) 0, apdu, ISO7816.OFFSET_CDATA, (byte)pinBytes.length);
        return apdu;
    }
    
    public static byte[] isPinUnlocked() {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x13;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte) 0x00;
        return apdu;
    }
    
    public static byte[] encrypt_cbc(byte[] bytes) {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH + bytes.length];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x51;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x01;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte)bytes.length;

        Util.arrayCopyNonAtomic(bytes, (short) 0, apdu, ISO7816.OFFSET_CDATA, (byte)bytes.length);
        return apdu;
    }
    
    public static byte[] decrypt_cbc(byte[] bytes) {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH + bytes.length];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x51;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x02;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte)bytes.length;

        Util.arrayCopyNonAtomic(bytes, (short) 0, apdu, ISO7816.OFFSET_CDATA, (byte)bytes.length);
        return apdu;
    }
    
    public static byte[] finalize_cbc() {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x52;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte) 0x00;
        return apdu;
    }
    
    public static byte[] init_cbc_encryption(byte[] iv) {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH + iv.length];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x50;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x01;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte) iv.length;

        Util.arrayCopyNonAtomic(iv, (short) 0, apdu, ISO7816.OFFSET_CDATA, (byte)iv.length);
        return apdu;
    }
    
    public static byte[] init_cbc_decryption(byte[] iv) {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH + iv.length];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x50;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x02;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte) iv.length;

        Util.arrayCopyNonAtomic(iv, (short) 0, apdu, ISO7816.OFFSET_CDATA, (byte)iv.length);
        return apdu;
    }
    
    public static byte[] testSecure(byte[] data) {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH + data.length];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x80;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte) data.length;

        Util.arrayCopyNonAtomic(data, (short) 0, apdu, ISO7816.OFFSET_CDATA, (byte)data.length);
        //Util.arrayCopyNonAtomic(buffer, offset, tmp, (short) 0, length);
        return apdu;
    }
    
    public static byte[] dhInit() {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x60;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte) 0x00;

        return apdu;
    }
    
    public static byte[] dhGet() {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x61;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte) 0x00;

        //Util.arrayCopyNonAtomic(buffer, offset, tmp, (short) 0, length);
        return apdu;
    }
    
    public static byte[] dhGet2() {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x65;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte) 0x00;

        //Util.arrayCopyNonAtomic(buffer, offset, tmp, (short) 0, length);
        return apdu;
    }
    
    public static String bytesToHex(byte[] data) {
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < data.length; i++) {
            buf.append(byteToHex(data[i]));
            buf.append(" ");
        }
        return (buf.toString());
    }
    
    public static byte[] dhSetY(byte[] data) {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH + (data.length-2)];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x62;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte) 0x00;
        

        Util.arrayCopyNonAtomic(data, (short) 0, apdu, ISO7816.OFFSET_P2, (short) data.length);
        System.out.println("apdu to send: "+bytesToHex(apdu));
        return apdu;
    }
    
    public static byte[] dhDoFinal() {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x63;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte) 0x00;

        //Util.arrayCopyNonAtomic(buffer, offset, tmp, (short) 0, length);
        return apdu;
    }
    
    public static byte[] dhTest() {
        byte apdu[] = new byte[TerminalManager.HEADER_LENGTH];
        apdu[TerminalManager.OFFSET_CLA] = (byte) 0xB0;
        apdu[TerminalManager.OFFSET_INS] = (byte) 0x64;
        apdu[TerminalManager.OFFSET_P1] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_P2] = (byte) 0x00;
        apdu[TerminalManager.OFFSET_LC] = (byte) 0x00;

        //Util.arrayCopyNonAtomic(buffer, offset, tmp, (short) 0, length);
        return apdu;
    }
}
