/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package jpass.smartcard;

import Applet.JPassApplet;
import Applet.SecureChannel;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Arrays;
import javacard.security.CryptoException;
import javax.smartcardio.ResponseAPDU;
import static jpass.smartcard.TerminalManager.byteToHex;

/**
 *
 * @author macbook
 */
public class JavaCard {
    TerminalManager cardManager = new TerminalManager();
    
    private static byte APPLET_AID[] = {(byte) 0x73, (byte) 0x69, (byte) 0x6D, (byte) 0x70, (byte) 0x6C, 
        (byte) 0x65, (byte) 0x61, (byte) 0x70, (byte) 0x70, (byte) 0x6C, (byte) 0x65, (byte) 0x74};
    private static byte SELECT_SIMPLEAPPLET[] = {(byte) 0x00, (byte) 0xa4, (byte) 0x04, (byte) 0x00, (byte) 0x0b, 
        (byte) 0x73, (byte) 0x69, (byte) 0x6D, (byte) 0x70, (byte) 0x6C,
        (byte) 0x65, (byte) 0x61, (byte) 0x70, (byte) 0x70, (byte) 0x6C, (byte) 0x65, (byte) 0x74};
    
    private static byte SUCCESS_CODE[] = {(byte) 0x90, (byte) 0x00};
    
    private enum processMode {
        ENCRYPT, DECRYPT
    }
    
    private SecureChannel secureChannel = null;
    
        
    boolean isSimulator = true;
    
    JavaCard(boolean isSimulator) throws Exception {
        this.isSimulator = isSimulator;
        if(!this.setupCard()) {
            throw(new Exception("Error while initalizing card"));
        }
        
        secureChannel = new SecureChannel();
        byte[] enc = {(byte) 0x10, (byte) 0x02, (byte) 0x03, (byte) 0x04, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04};
        
        secureChannel.setSessionKey(enc, (short)0);
    }
    
    /*
     *  Call this when you are done with the card.
    */    
    public void tearDown() {
        this.cleanup(this.isSimulator);
    }
    
    public static String bytesToHex(byte[] data) {
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < data.length; i++) {
            buf.append(byteToHex(data[i]));
            buf.append(" ");
        }
        return (buf.toString());
    }
    
    private byte[] sendAPDU(byte[] apdu) throws Exception {
        ResponseAPDU response = this.sendAPDU(secureChannel.encryptSend(apdu), this.isSimulator);
        
        byte[] data = secureChannel.decryptResponse(response);
        if (data.length >= 2) {
            byte[] code = Arrays.copyOfRange(data, data.length -  2, data.length); 
            
            if(Arrays.equals(code, SUCCESS_CODE)) {
               return Arrays.copyOfRange(data, 0, data.length - 2);
            }
            throw(new CardException("Incorrect APDU"));
        }
        throw(new CardException("Incorrect response"));
    }
    
    private byte[] sendDHAPDU(byte[] apdu) throws Exception {
        ResponseAPDU response = this.sendAPDU(apdu, this.isSimulator);
        
        byte[] data = response.getBytes();
        if (data.length >= 2) {
            byte[] code = Arrays.copyOfRange(data, data.length -  2, data.length); 
            if(Arrays.equals(code, SUCCESS_CODE)) {
               return Arrays.copyOfRange(data, 0, data.length - 2);
            }
            throw(new CardException("Incorrect APDU"));
        }
        throw(new CardException("Incorrect response"));
    }
    
    
    private boolean setupCard() {
        return this.setupCard(this.isSimulator);
    }
    
    private boolean setupCard(boolean isSimulator) {
        if(isSimulator){
            byte[] installData = new byte[10];
            cardManager.prepareLocalSimulatorApplet(APPLET_AID, installData, JPassApplet.class);
            
            return true;
        } else {
            try {
                boolean status = cardManager.ConnectToCard();
                cardManager.sendAPDU(SELECT_SIMPLEAPPLET);
                return status;      
            } catch (Exception ex) {
                return false;
            }
        }
    }
    
    private void cleanup(boolean isSimulator) {
        try {
            cardManager.DisconnectFromCard();
        } catch (Exception ex) { }
    }
    
    private ResponseAPDU sendAPDU(byte[] apdu, boolean isSimulator) throws Exception {
        if(isSimulator) {
            return new ResponseAPDU(cardManager.sendAPDUSimulator(apdu)); 
        } else {
            return cardManager.sendAPDU(apdu);
        }
    }
    
    public boolean isPinUnlocked() {
        try { 
            this.sendAPDU( APDUFactory.isPinUnlocked() ); 
        } 
        catch (Exception ex) { return false; }
        return true;
    }
    
    public boolean setPin(byte[] pin) {
        try { 
            this.sendAPDU( APDUFactory.setPin(pin) ); 
        } 
        catch (Exception ex) { return false; }
        return true;
    }
    
    public boolean verifyPin(byte[] pin) {
        try { 
            this.sendAPDU( APDUFactory.verifyPin(pin) ); 
        } 
        catch (Exception ex) { return false; }
        return true;
    }
    
    public byte[] encryptBytes(byte[] IV, byte[] data) throws Exception {
        InitEncryption(IV);
        return this.processCipher(data, processMode.ENCRYPT);
    }
    
    public byte[] decryptBytes(byte[] IV, byte[] data) throws Exception {
        InitDecryption(IV);
        return this.processCipher(data, processMode.DECRYPT);
    }
    
    private byte[] processCipher(byte[] data, processMode mode) throws IOException, Exception {
        int length = data.length;
        int bytesRead = 0;
        int blockLength = 16;
        ByteArrayOutputStream encrypted_file_stream = new ByteArrayOutputStream( );
        // Encrypt it
        while( bytesRead != length ){
            byte[] block;
            int bytesLeft = length - bytesRead;
            
            if( bytesLeft > blockLength ) {
                block = Arrays.copyOfRange(data, bytesRead, blockLength + bytesRead);
                bytesRead += blockLength;
            } else {
                block = Arrays.copyOfRange(data, bytesRead, bytesLeft + bytesRead);
                bytesRead += bytesLeft;
            }
            
            byte[] apdu =  (mode == processMode.ENCRYPT) ? APDUFactory.encrypt_cbc(block) : APDUFactory.decrypt_cbc(block);
                        
            byte[] encrypted_block = this.sendAPDU( apdu );
            encrypted_file_stream.write(encrypted_block);
        }
        System.out.println("Finalizing");
        byte[] encrypted_block = this.sendAPDU( APDUFactory.finalize_cbc() );
        encrypted_file_stream.write(encrypted_block);
        byte[] encrypted_file = encrypted_file_stream.toByteArray( );
        return encrypted_file;
    }
    
    private void InitEncryption(byte[] IV) throws CryptoException, Exception {
        this.sendAPDU( APDUFactory.init_cbc_encryption(IV) );
    }
    
    private void InitDecryption(byte[] IV) throws CryptoException, Exception {
        this.sendAPDU( APDUFactory.init_cbc_decryption(IV) );
    }
    
    public byte[] testSecure(byte[] apdu) {
        byte[] response = {(byte) 0x00};
        try { 
            response = this.sendAPDU( APDUFactory.testSecure(apdu) ); 
        }
        catch (Exception ex) { return response; }
        return response;
    }
    
    // diffie hellman instructions
    public boolean dhInit() throws CryptoException, Exception {
        this.sendDHAPDU( APDUFactory.dhInit() );
        return true;
    }
    
    public byte[] dhGet() throws CryptoException, Exception {
        return this.sendDHAPDU( APDUFactory.dhGet() );
    }
    
    public byte[] dhGet2() throws CryptoException, Exception {
        return this.sendDHAPDU( APDUFactory.dhGet2() );
    }
    
    public boolean dhSetY(byte[] apdu) throws CryptoException, Exception {
        this.sendDHAPDU( APDUFactory.dhSetY(apdu) );
        return true;
    }
    
    public byte[] dhDoFinal() throws CryptoException, Exception {
        return this.sendDHAPDU( APDUFactory.dhDoFinal() );
    }
    
    public byte[] dhTest() throws CryptoException, Exception {
        return this.sendDHAPDU( APDUFactory.dhTest() );
    }
    
}